@@ -1,7 +1,7 @@
 #include <stdio.h>
 #include "../kbtree.h"
 
-typedef int num;
+typedef intptr_t num;
 
 static num numcmp(num a, num b)
 {
@@ -16,9 +16,8 @@ int main()
     for (num n=0;n<=1000;n++)
     {
         kbtree_t(num) *elm = kb_init(num, KB_DEFAULT_SIZE);
-        elm = kb_init(num, KB_DEFAULT_SIZE);
         for (int i=0; i<n; i++)
-          kb_put(num, elm, i*2+1);
+            kb_put(num, elm, i*2+1);
 
         for (num k=0; k<=n*2; k++)
         {
@@ -31,7 +30,7 @@ int main()
                 num l = (k + 1) | 1;
                 if (j != l)
                 {
-                    printf("got %d wanted %d\n", j, l);
+                    printf("got %zd wanted %zd\n", j, l);
                     return 1;
                 }
             }
